//String.cpp             Implementation Of String Class
#include  <stdio.h>      
#include  <string.h>
#include  "Strings.h"


/* ****************************************************************************/
// String::String()  This is an uninitialized String Constructor.  It is 
// called when a declaration of a String such as this occurs...
//  
// String s1;
//  
// All it does is assign NULL to the pStrBuffer member. 
/* ****************************************************************************/
String::String()
{
 puts("Entering Uninitialized String Constructor!");
 puts("Leaving Uninitialized String Constructor!\n");   
 pStrBuffer=NULL;
}



/* ************************************************************************** */
// String& String::operator=(const char c)   This is an overloaded operator=
// called when a String is assigned a char like so...
//
// String s1, s2;
//
// s1='F';
// s2=114;  //'r'
/* ************************************************************************** */
String& String::operator=(const char c)  //Overloaded operator = for assigning a
{                                        //character to a String
 puts("Entering operator=(const char)");  
 if(this->pStrBuffer)            //If String already is storing something delete
    delete [] this->pStrBuffer;  //it.  Then allocate a new buffer of two bytes;
 pStrBuffer=new char[2];         //one for the char and one for the obligatory
 this->pStrBuffer[0]=c;          //NULL.  Then, using base pointer offset 
 this->pStrBuffer[1]='\0';       //notation, copy char to offset zero and the 
 puts("Leaving operator=(const char)");  
                                 //NULL right afer it.
 return *this;
}



/* ************************************************************************** */
//  This overloaded operator= is called when a String is assigned a char*.  In
//  other words, something like this...
//  
//  String s1;
//  
//  s1="Hello, World!"  
//  
//  or
//  
//  char* pStr="Hello, World!";
//  s1=pStr; 
//  
//  The 1st thing the function does is check to see if this->pStrBuffer isn't 
//  NULL.  If it isn't then the String is holding some previous String the user
//  doesn't want anymore.  In that case delete [] is called on the String.  In 
//  any case a new buffer is then allocated of a length equal to the length of 
//  the char* pStr passed in plus an extra byte for the null terminator.  Then 
//  the passed in parameter string is copied to the newly acquired buffer.  
//  While this points to a String, *this is a String, and now since the char* 
//  has been transfered to the String's storage, we simply return *this.
/* ************************************************************************** */
String& String::operator=(const char* pStr)
{
 puts("Entering operator=(const char* pStr)");   
 if(this->pStrBuffer)
    delete pStrBuffer;
 pStrBuffer=new char[strlen(pStr)+1];
 strcpy(pStrBuffer,pStr);
 puts("Leaving operator=(const char* pStr)");  
    
 return *this;
}



/* ************************************************************************** */
// String& String::operator=(const String& strRight)  Overloaded operator= for
// assigning a second string to a first.  It is triggered by a statement such as
// this
// 
// String s1, s2;
// 
// s1 = "PowerBASIC Put The Power In BASIC";
// s2 = s1; 
//
// The only thing tricky here is the top if.  If you attempt to assign a String
// to itself, e.g.,
//
// s1 = s1;
//
// what will happen is the String will delete itself, then attempt to copy the
// deleted String back to a zero length buffer.  The only sane alternative is to
// return the existing String 'as is', as you can see by the return *this. 
//
// Otherwise, the function deletes [] the existing buffer, allocates a new one
// adequate to hold the parameter's String, then copies the latter to the new
// buffer.
/* ************************************************************************** */
String& String::operator=(const String& strRight) //Overloaded operator = for 
{                                                 //assigning another String to 
 puts("Entering operator=(const String& strRight)");  
 printf("  strRight.pStrBuffer = %s\n",strRight.pStrBuffer);
 if(this==&strRight)                              //a String
 {
    puts("Leaving operator=(const String& strRight) In Early Exit\n");     
    return *this;
 }   
 delete [] this->pStrBuffer;
 pStrBuffer=new char[strlen(strRight.pStrBuffer)+1];
 strcpy(pStrBuffer,strRight.pStrBuffer);
 puts("Leaving operator=(const String& strRight)\n");  

 return *this;
}



/* ************************************************************************** */
// String& String::operator+(const char ch)  Adds a character to a String.  If
// you have this...
//
// String s1;
//
// s1 = "ABC";
// s1 = s1 + 'D';
//
// ...then this overloaded operator= String member function will be called.
/* ************************************************************************** */
String& String::operator+(const char ch) //Overloaded operator + (Adds char in 
{                                        //String)
 unsigned int iLen=0;
 char* pNew;

 puts("Entering operator+(const char ch)");
 printf("  ch   = %c\n",ch);
 if(this->pStrBuffer) //1st term (pStrBuffer) isn't empty (NULL)
 {
    iLen=strlen(this->pStrBuffer);  //find out length of existing String
    pNew=new char[iLen+2];          //allocate new buffer with two extra bytes
    strcpy(pNew,this->pStrBuffer);  //copy existing String to new buffer
    delete [] this->pStrBuffer;     //delete [] existing String
    this->pStrBuffer=pNew;          //set 'this' String to new buffer address
    this->pStrBuffer[iLen]=ch;      //add char to end of existing String
    this->pStrBuffer[iLen+1]='\0';  //add NULL byte right after that
 }
 else                 //1st string (this->pStrBuffer) is empty (NULL)
 {
    pStrBuffer=new char[2];
    this->pStrBuffer[0]=ch;
    this->pStrBuffer[1]='\0';
 }
 printf("  iLen = %u\n",iLen);
 puts("Leaving operator+(const char ch)\n");

 return *this;
}



/* ************************************************************************** */
// This function simply returns the pStrBuffer private char* address of the
// String.  It is useful for example in outputing the string to a file or
// display with the printf function. 
/* ************************************************************************** */
char* String::lpStr()
{
 return this->pStrBuffer;
}



/* ************************************************************************** */
// This is the String Destructor.  It is called when a String is being 
// destroyed.  In such cases it is considered good practice to set any class
// buffers to 0 so the same address does't accidentally get deleted twice.  
// Calling delete on a NULL pointer is OK; calling it twice on a non null 
// pointer isn't OK, because there may be something else at that address.
/* ************************************************************************** */
String::~String()    //String Destructor
{
 delete pStrBuffer;
 pStrBuffer=NULL;
}
  
